import redis
from .config import config

__client_map = {}


def get_redis_client(db=0, label='redis'):
    """
    @rtype: StrictRedis
    """
    redis_host = config(label,'host')
    redis_port = config(label,'port')
    redis_pass = config(label,'pass')
    if len(redis_pass) == 0:
        redis_pass = None
    global __client_map
    if label not in __client_map:
        redis_pool = redis.ConnectionPool(host=redis_host, port=redis_port, password=redis_pass, db=db)
        clients = redis.Redis(connection_pool=redis_pool)
        __client_map[label] = clients

    return __client_map[label]



import pika
from .config import config


class BaseRabbit(object):

    def __init__(self, label='rabbit'):
        host = config(label, 'host')
        port = int(config(label, 'port'))
        user = config(label, 'user')
        password = config(label, 'password')
        credentials = pika.PlainCredentials(user, password)
        self._connection = pika.BlockingConnection(pika.ConnectionParameters(host, port, '/', credentials))
        self._channel = self._connection.channel()
        self._caller = None


class RabbitProducer0(BaseRabbit):

    def __init__(self, label='rabbit'):
        super(RabbitProducer0,self).__init__(label)

    def send(self, topic, message):
        self._channel.queue_declare(queue=topic, durable=True)
        self._channel.basic_publish(exchange='',
                                    routing_key=topic,
                                    body=message,
                                    properties=pika.BasicProperties(
                                        delivery_mode=2,  # 使得消息持久化
                                    ))


class RabbitConsumer(BaseRabbit):

    def __init__(self, topic, label='rabbit'):
        self._topic = topic
        super(RabbitConsumer, self).__init__(label)

    def callback(self, ch, method, properties, body):
        self._caller(body)
        ch.basic_ack(delivery_tag=method.delivery_tag)

    def execute(self, caller, prefetch_count=2):
        self._caller = caller
        channel = self._connection.channel()
        channel.basic_qos(prefetch_count=prefetch_count)
        channel.basic_consume(self.callback,
                              queue=self._topic,
                              no_ack=False)
        channel.start_consuming()



import numpy
import json
import datetime
import time
from dateutil import parser


def dump_json(data):
    return json.dumps(data, ensure_ascii=False)


def load_json(data):
    try:
        return json.loads(data)
    except:
        return None


def parse_date(str):
    return parser.parse(str)


def timestamp_to_str(time, format='%Y-%m-%d'):
    return datetime.datetime.fromtimestamp(time).strftime(format)


def get_current_millisecond():
    return int(round(time.time() * 1000))






import sense_core as sd
import os
import time


def test_object_id():
    print(sd.generate_object_id())


def test_log_info():
    sd.log_init_config('core', '../logs','',True)
    sd.log_info('logxxxxx')


def test_log_error():
    sd.log_init_config('core', '../logs')
    sd.log_error('error xxx')


def test_log_error_monit():
    sd.log_init_config('core', '../logs', 'rabbit1')
    sd.log_error('error xxx')


def test_log_exception():
    sd.log_init_config('core', '../logs','rabbit1')
    try:
        print(5 / 0)
    except Exception as ex:
        sd.log_exception(ex)


def test_config():
    print(sd.config('db_stock', 'dbms'))
    print(sd.config('database'))


def consume_message(msg):
    sd.log_info(msg)
    # sd.log_info('consumer ' + str(os.getpid()) + ' msg=' + msg)
    time.sleep(2)


def test_rabbit_produce():
    sd.log_init_config('core', '../logs', False)
    producer = sd.RabbitProducer()
    for i in range(1, 100):
        producer.send('test3', 'helloo=%d' % i)


def test_kafka_consumer():
    sd.log_init_config('core', '../log', False)
    consumer = sd.RabbitConsumer('test3')
    consumer.execute(consume_message)



from .object_id import generate_object_id
from .log import log_init_config, log_info, log_error, log_exception, raise_exception
from .utils import *
from .django import build_model_list
from .config import config
from .es import get_es_client, BaseSearcher, BaseIndexer
from .redis import get_redis_client
from .queue import RabbitProducer
from .queue import RabbitConsumer
